# AUTOGENERATED! DO NOT EDIT! File to edit: ../../../nbs/data_structure/timeseries/timeseries.ipynb.

# %% auto 0
__all__ = ['B_TsOption', 'V_TsOption', 'n_TsOption', 'ts_options', 'TsOption', 'set_ts_option']

# %% ../../../nbs/data_structure/timeseries/timeseries.ipynb 0
from typing_extensions import TypedDict
from xarray import DataArray

# %% ../../../nbs/data_structure/timeseries/timeseries.ipynb 1
class TsOption(TypedDict):
    title: str
    units: str
    subtitle: str
    legend_names: list[str]


B_TsOption = TsOption(
    title="$B$",
    units="nT",
    subtitle="[nT LMN]",
    legend_names=[r"$B_l$", r"$B_m$", r"$B_n$", r"$B_{total}$"],
)

V_TsOption = TsOption(
    title="$V$",
    subtitle="[km/s LMN]",
    units="km/s",
    legend_names=[r"$V_l$", r"$V_m$", r"$V_n$", r"$V_{total}$"],
)

n_TsOption = TsOption(title="$n_i$", units="cm$^{-3}$")

# %% ../../../nbs/data_structure/timeseries/timeseries.ipynb 2
ts_options = {
    "B": B_TsOption,
    "V": V_TsOption,
    "n": n_TsOption,
}


def set_ts_option(da: DataArray, type="B", options_dict=ts_options):
    da.attrs["long_name"] = options_dict[type]["title"]
    da.attrs["units"] = options_dict[type]["units"]
    if "v_dim" in da.dims:
        da["v_dim"] = options_dict[type]["legend_names"]
    return da
